import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Color;

/**
 * A Menu
 * 
 * @author Taylor Born
 * @version March 2011
 */
public class Menu  extends Actor
{
    static Font font = new Font("New Times Roman", Font.PLAIN, 12);

    private Point home;
    private Point lastMouse = new Point(-25, -25);
    private ArrayList<MenuItem> items = new ArrayList<MenuItem>();
    private String active = "";
    private String itemPressed = null;

    /**
     * Create the Menu with the initial Menu "headings".
     * @param menuItems This first list are the Menu items that are always displayed on the "top" 
     * bar. Menu items can be added to this "top" bar with the adding methods, just don't need 
     * parent Menu items listed.
     */
    public Menu(ArrayList<String> menuItems)
    {
        for (String s : menuItems)
            items.add(new MenuItem(s));
    }
    
    public void act() 
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mouseMoved(null) || Greenfoot.mouseDragged(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        
        int height = 12;
        
        int width = 0;
        
        int barWidth = 0;
        for (MenuItem mi : items)
            barWidth += mi.getWidth();
        
        if (active.length() == 0)
            width = barWidth;
        else
        {
            for (int i = 0; i < items.size(); i++)
                if (items.get(i).getName().equals(active.substring(0, active.indexOf("/"))))
                {
                    width += items.get(i).getRunningWidth(active.substring(active.indexOf("/") + 1), true);
                    height += 12 + items.get(i).getRunningHeight(active.substring(active.indexOf("/") + 1));
                    break;
                }
                else
                    width += items.get(i).getWidth();
            if (barWidth > width)
                width = barWidth;
        }
        
        setLocation(home.getX() + (width - barWidth) / 2, home.getY() + (height - 12) / 2);
        Point offsetMouse = new Point(lastMouse.getX() - (getX() - (width / 2)), lastMouse.getY() - (getY() - (height / 2)));
        
        boolean clicked = Greenfoot.mouseClicked(null);
        if (Greenfoot.mouseClicked(this) && offsetMouse.getX() > 0 && offsetMouse.getX() < width && offsetMouse.getY() > 0 && offsetMouse.getY() < 12)
        {
            int runX = 0;
            for (int i = 0; i < items.size(); i++)
                if (runX + items.get(i).getWidth() > offsetMouse.getX())
                {
                    if (active.startsWith(items.get(i).getName() + "/"))
                        active = "";
                    else
                        active = items.get(i).getName() + "/";
                    break;
                }
                else
                    runX += items.get(i).getWidth();
            clicked = false;
        }
        
        GreenfootImage pic = new GreenfootImage(width + (width % 2 != 0 ? 2 : 1), height + 1);
        pic.setFont(font);
        
        int runWidth = 0;
        for (MenuItem barItem : items)
        {
            pic.setColor(Color.BLACK);
            if (active.length() != 0 && offsetMouse.getX() > runWidth && offsetMouse.getX() <= runWidth + barItem.getWidth() && offsetMouse.getY() > 0 && offsetMouse.getY() <= 12)
            {
                active = barItem.getName() + "/";
                if (clicked)
                    itemPressed = active;
            }
            if (active.startsWith(barItem.getName() + "/"))
            {
                String s = active.substring(active.indexOf("/") + 1);
                int runningX = runWidth;
                int runningY = 12;
                MenuItem item = barItem;
                boolean sneakpeek = false;
                if (s.length() == 0 && item.getItems().size() > 0)
                    sneakpeek = true;
                String address = item.getName() + "/";
                while (s.length() != 0 || sneakpeek)
                {
                    int indexOfNext = -1;
                    for (int i = 0; i < item.getItems().size(); i++)
                    {
                        pic.setColor(Color.WHITE);
                        if (!sneakpeek && item.getItems().get(i).getName().equals(s.substring(0, s.indexOf("/"))))
                        {
                            indexOfNext = i;
                            pic.setColor(Color.CYAN);
                        }
                        int w = item.getLargestChild();
                        if (offsetMouse.getX() > runningX && offsetMouse.getX() <= runningX + w && offsetMouse.getY() > runningY + i * 12 && offsetMouse.getY() <= runningY + i * 12 + 12)
                        {
                            active = address + item.getItems().get(i).getName() + "/";
                            if (clicked)
                                itemPressed = active;
                            pic.setColor(Color.CYAN);
                        }
                        pic.fillRect(runningX, runningY + i * 12, w, 12);
                        if (item.getItems().get(i).getItems().size() > 0)
                        {
                            pic.setColor(Color.GRAY);
                            int[] xs = new int[3];
                            xs[0] = runningX + w - 8;
                            xs[1] = runningX + w - 1;
                            xs[2] = runningX + w - 8;
                            int[] ys = new int[3];
                            ys[0] = runningY + i * 12 + 2;
                            ys[1] = runningY + i * 12 + 6;
                            ys[2] = runningY + i * 12 + 10;
                            pic.fillPolygon(xs, ys, 3);
                        }
                        pic.setColor(Color.BLACK);
                        pic.drawRect(runningX, runningY + i * 12, w, 12);
                        pic.drawString(item.getItems().get(i).getName(), runningX + 2, runningY + i * 12 + 11);
                    }
                    if (!sneakpeek)
                    {
                        s = s.substring(s.indexOf("/") + 1);
                        if (indexOfNext != -1)
                        {
                            runningX += item.getLargestChild();
                            runningY += indexOfNext * 12;
                            item = item.getItems().get(indexOfNext);
                            address += item.getName() + "/";
                            if (s.length() == 0 && item.getItems().size() > 0)
                                sneakpeek = true;
                        }
                    }
                    else
                    {
                        s = "";
                        sneakpeek = false;
                    }
                }
                pic.setColor(Color.CYAN);
            }
            else
                pic.setColor(Color.WHITE);
            pic.fillRect(runWidth, 0, barItem.getWidth(), 12);
            pic.setColor(Color.BLACK);
            pic.drawRect(runWidth, 0, barItem.getWidth(), 12);
            pic.drawString(barItem.getName(), runWidth + 2, 11);
            runWidth += barItem.getWidth();
        }
        if (clicked)
            active = "";
        setImage(pic);
    }
    
    /**
     * The action listener for the Menu.
     * @return String representation of what Menu item was last clicked in the format of the
     * path getting to it. For example if an "Arrow" Menu item was clicked: "Insert/Shape/Arrow/"
     */
    public String getItemPressed()
    {
        String s = itemPressed;
        itemPressed = null;
        return s;
    }
    
    /**
     * Add an aditional Menu item to the Menu.
     * @param address Specify the path to the location to add something, with the item being 
     * added listed at the end. For example, to add "Square" in the list at "Insert/Shape/", 
     * input this: "Insert/Shape/Square/". Note that the parent Menu items must exist.
     * @see #addItems(String, ArrayList)
     */
    public void addItem(String address)
    {
        if (!address.contains("/"))
            items.add(new MenuItem(address));
        else
        {
            for (MenuItem mi : items)
                if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                {
                    mi.add(address.substring(address.indexOf("/") + 1));
                    break;
                }
        }
    }
    
    /**
     * Add multiple aditional Menu items to the Menu into a single address.
     * @param address The address for where to add the Menu items. For example, "Insert/Shape/" 
     * will specify to add the Menu items under "Shape". Note that these parent Menu items must exist.
     * @param menuItems The list of Menu items to add to the Menu. For example, "Circle/" will add
     * a Menu item with the name "Circle" at the location specified by the address param.
     * @see #addItem(String)
     */
    public void addItems(String address, ArrayList<String> menuItems)
    {
        for (String str : menuItems)
            addItem(address + str);
    }
    
    public void addedToWorld(World world)
    {
        home = new Point(getX(), getY());
    }
}

class MenuItem
{
    private ArrayList<MenuItem> items = new ArrayList<MenuItem>();
    private String name;
    
    public MenuItem(String n)
    {
        name = n;
    }
    
    public void add(String s)
    {
        if (!s.contains("/"))
            items.add(new MenuItem(s));
        else
        {
            for (MenuItem mi : items)
                if (mi.getName().equals(s.substring(0, s.indexOf("/"))))
                {
                    mi.add(s.substring(s.indexOf("/") + 1));
                    break;
                }
        }
    }
    
    public int getWidth()
    {
        Graphics2D graphics = (new GreenfootImage(1, 1)).getAwtImage().createGraphics();
        graphics.setFont(Menu.font);
        FontMetrics fm = graphics.getFontMetrics();
        return fm.charsWidth(name.toCharArray(), 0, name.length()) + 4;
    }
    
    public int getRunningWidth(String s, boolean first)
    {
        int w = getLargestChild();
        if (s.length() != 0)
            for (MenuItem mi : items)
                if (mi.getName().equals(s.substring(0, s.indexOf("/"))))
                    return w + mi.getRunningWidth(s.substring(s.indexOf("/") + 1), false);
        return w;
    }
    public int getLargestChild()
    {
        int n = 0;
        for (MenuItem mi : items)
            if (mi.getWidth() + (mi.getItems().size() > 0 ? 10 : 0) > n)
                n = mi.getWidth() + (mi.getItems().size() > 0 ? 10 : 0);
        return n;
    }
    
    public int getRunningHeight(String s)
    {
        if (items.size() == 0)
            return 0;
        int h = (items.size() - 1) * 12;
        if (s.length() != 0)
            for (MenuItem mi : items)
                if (mi.getName().equals(s.substring(0, s.indexOf("/"))))
                    return h + mi.getRunningHeight(s.substring(s.indexOf("/") + 1));
        return h;
    }
    
    public ArrayList<MenuItem> getItems()
    {
        return items;
    }
    
    public String getName()
    {
        return name;
    }
}