/**
 * A Point
 * 
 * @author Taylor Born
 * @version March 2010
 */
public class Point  
{
    private int x, y;
    
    /**
     * Create a new Point with new values.
     * @param x The x value.
     * @param y The y value.
     */
    public Point(int x, int y)
    {
        this.x = x;
        this.y = y;
    }
    
    /**
     * Create a new Point from the values of another Point.
     * @param p The Point whose values will be used to set the new Point's values.
     */
    public Point(Point p)
    {
        x = p.getX();
        y = p.getY();
    }
    
    /**
     * Get the x value.
     * @return The x value.
     */
    public int getX()
    {
        return x;
    }
    
    /**
     * Get the y value.
     * @return The y value.
     */
    public int getY()
    {
        return y;
    }
    
    /**
     * Change the x and y values by the given amounts.
     * @param xChange The value to be added to the x value.
     * @param yChange The value to be added to the y value.
     */
    public void move(int xChange, int yChange)
    {
        x += xChange;
        y += yChange;
    }
    
    /**
     * Determine if another Object is equal to this one.
     * @return Whether the given Object is a Point and its values equal to this Point values.
     */
    public boolean equals(Object other)
    {
        if (other != null && other instanceof Point)
            return x == ((Point)other).getX() && y == ((Point)other).getY();
        return false;
    }
}